/***************************************************************
 *                    simula.plus@cemes.fr                     *
 *                   GNU/linux version 3.2.0                   *
 *            software under General Public License            *
 ***************************************************************
 * copyright © 2005 CREUSE Emmanuel
 * copyright © 2012 COLLARD Christophe
 * copyright © 2004,2012 Centre National de la Recherche Scientifique
 * copyright © 2004 Arts et Métiers ParisTech
 * copyright © 2004 Université de Valenciennes et du Hainaut Cambrésis
 * copyright © 2004 Laboratoire de Physique et Mécanique des Matériaux (LPMM - UMR 7554)
 * copyright © 2004 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV - EA 4015)
 * copyright © 2012 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
 ***************************************************************/

/*
    rt02d-test belongs to Mathematical Object Libraries (FEMOL++)
    FEMOL++ is part of Simula+

    Simula+ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Simula+ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Simula+; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#ifndef __cplusplus
#error Must use C++ for the type rt02d
#endif

#ifndef __rt02d_test_hpp
#define __rt02d_test_hpp

#ifndef __assert_h
#include <assert.h>
#endif

#ifndef __time_h
#include <time.h>
#endif

#ifndef __vectors_hpp
#include "MOL++/vectors.hpp"
#endif

#ifndef __matrix_hpp
#include "MOL++/matrix.hpp"
#endif

#ifndef __spmatrix_hpp
#include "MOL++/spmatrix.hpp"
#endif

#ifndef __rt02d_hpp
#include "FEMOL++/elements/rt02d.hpp"
#endif

#ifndef __affiche_hpp
#include "tests/affiche.hpp"
#endif

long double p = 1.5;
long double vareps = 0.05;

long double r2 (long double x, long double y)
{
  return pow (x - 0.5, 2) + pow (y - 0.5, 2);
}

long double f (long double x, long double y)
{
  long double d2 = r2 (x,y);
  if (d2 <= 0.25)
    return pow (2/vareps, p-1) * pow (sqrt (d2) * (vareps + 0.25 - d2), p-2) * ((vareps + 0.25 - d2) * (p-2 * (p-1) * d2 / vareps) - 2* (p-1) * d2) * exp (-(p-1) * d2 / vareps);
  else return 0;
}

long double u (long double x, long double y)
{
  long double d2 = r2 (x,y);
  if (d2 <= 0.25)
    return (0.25 - d2) * exp(-d2 / vareps);
  else
    return 0;
}

long double dudx (long double x, long double y)
{
  long double d2 = r2 (x,y);
  if (d2 <= 0.25)
    return (1 - 2 * x) * exp (-d2 / vareps) * (vareps + 0.25 - d2) / vareps;
  else return 0;
}

long double dudy (long double x, long double y)
{
  long double d2 = r2 (x,y);
  if (d2 <= 0.25)
    return (1 - 2 * y) * exp (-d2 / vareps) * (vareps + 0.25 - d2) / vareps;
  else
    return 0;
}

long double NormGradu (long double x, long double y)
{
  long double d2 = r2 (x,y);
  if (d2 <= 0.25)
    return 2 * sqrt (d2) * exp (-d2 / vareps) * (vareps + 0.25 - d2) / vareps;
  else
    return 0;
}

long double px (long double x, long double y)
{
  long double d2 = r2 (x,y);
  if (d2 <= 0.25)
    return ((1 - 2*x) * exp (-d2 / vareps) * (vareps + 0.25 - d2) / vareps) * pow (NormGradu(x,y), p-2);
  else
    return 0;
}

long double py (long double x, long double y)
{
  long double d2 = r2 (x,y);
  if (d2 <= 0.25)
    return ((1 - 2*y) * exp (-d2 / vareps) * (vareps + 0.25 - d2) / vareps) * pow (NormGradu(x,y), p-2);
  else
    return 0;
}

using namespace std;
using namespace mol;
using namespace femol;


//=========================
int test_rt02d (int detail)
//=========================
{
  int result = 1;

  if (detail < 0) 
    { cout << "============================================================== \n";
      cout << blue << "                      rt02d test skipped" << reset;
      cout << "============================================================== \n";
      return result;
    }

  rt02d<long double> Th ("data/2.txt", "data/21.txt");
  rt02d<long double> Th2 ("data/22.txt");
  rt02d<long double> Uh ("data/8.txt", "data/81.txt");
  rt02d<long double> Uh2 ("data/82.txt");

  rt02d<long double> Th3, Th4, Uh3, Uh4;

  if (detail) affiche ("operator ==", Th == Th2  &&  Uh == Uh2  &&  !(Uh == Th));
  else result *= (Th == Th2  &&  Uh == Uh2  &&  !(Uh == Th));

  if (detail) affiche ("operator !=", Th != Uh  &&  !(Th != Th2));
  else result *= (Th != Uh  &&  !(Th != Th2));

  rt02d<long double> Vh (Uh);
  if (detail) affiche ("copy constructor", Vh == Uh);
  else result *= (Vh == Uh);

  Th3 = Th;
  Th4 = Th2;
  Uh3 = Uh;
  Uh4 = Uh2;
  if (detail) affiche ("operator =", Th==Th3 && Th4==Th2 && Uh==Uh3 && Uh4==Uh2);
  else result *= (Th==Th3 && Th4==Th2 && Uh==Uh3 && Uh4==Uh2);

  int NbIntPts = 2;
  long double q = p / (p-1);
  rt02d<long double> Wh ("data/4.txt", "data/41.txt");
  long double erreur_p, erreur_u, erreur, erreur_max;
  long double eps = 1e-06;
  int nbiter = 0;
  //  spmatrix<long double> A (Wh.GlobalDofNumber(), Wh.GlobalDofNumber(), 5);
  vector<long double> B (Wh.GlobalDofNumber());
  vector<long double> Lambda (Wh.GlobalDofNumber());
  vector<long double> X0 (Wh.GlobalDofNumber());
  vector<long double> U (Wh.nbelement());
  vector<long double> Pmp1 (Wh.GlobalDofNumber(), true, 1);
  vector<long double> Pm (Wh.GlobalDofNumber());

  for (int indice_boucle=1; ((Pmp1-Pm).norm()>eps); indice_boucle++)
    { spmatrix<long double> A (Wh.GlobalDofNumber(), Wh.GlobalDofNumber(), 5);
      Pm = Pmp1;
      for (int i=1; i<=Wh.GlobalDofNumber(); i++)
	B[i] = 0;
      Wh.Assemble (A, B, f, Pm, q, NbIntPts);
      Wh.Dirichlet (A, B, u, p);
      Lambda = conjugate_gradient (A, B, X0, nbiter, precision);
      Wh.Compute_u (U, Lambda, f, Pm, q, NbIntPts);
      Wh.Compute_p (Pmp1, U,Lambda, Pm, q, NbIntPts);
      erreur_p = Wh.error_p (Pmp1, px, py, q, NbIntPts);
      erreur_u = Wh.error_u (U, p, u, NbIntPts);
    }

  long double erreur_u_hard = 0.0227015;
  long double erreur_p_hard = 0.308528810739517;

  if (detail) cout << "Resolution of p-laplacian with mixed hybrid method... :" << "\n";
  if (detail) affiche ("u error", abs (erreur_u - erreur_u_hard) < 1e-06);
  else result *= (abs (erreur_u - erreur_u_hard) < 1e-06);

  if (detail) affiche ("p error", abs (erreur_p - erreur_p_hard) < 1e-06);
  else result *= (abs (erreur_p - erreur_p_hard) < 1e-06);

  long double ETA1 = Th.eta_1 (Pm,p,2);
  long double ETA2 = Th.eta_2 (f,q,2);
  long double ETA1_hard = 0.160702243447;
  long double ETA2_hard = 3.15213227272;

  if (detail) cout << "A posteriori error estimator... :" << "\n" ;

  if (detail) affiche ("Estimator eta_1", abs (ETA1 - ETA1_hard) < 1e-08);
  else result *= (abs (ETA1 - ETA1_hard) < 1e-06);

  if (detail) affiche ("Estimator eta_2", abs (ETA2 - ETA2_hard) < 1e-08);
  else result *= (abs (ETA2 - ETA2_hard) < 1e-06);

  cout << "============================================================== \n";
  if (result) cout << green << "                      rt02d test passed" << reset;
  else cout << red << "                      rt02d test failed" << reset;
  cout << "============================================================== \n";

  return result;
}


#endif
